#!/usr/bin/env python
#
# $Id: cvsignore.py,v 1.1 2003/01/18 14:15:29 doughellmann Exp $
#
# Copyright 2003 Doug Hellmann.
#
#
#                         All Rights Reserved
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby
# granted, provided that the above copyright notice appear in all
# copies and that both that copyright notice and this permission
# notice appear in supporting documentation, and that the name of Doug
# Hellmann not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission.
#
# DOUG HELLMANN DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
# NO EVENT SHALL DOUG HELLMANN BE LIABLE FOR ANY SPECIAL, INDIRECT OR
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
# OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
# NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
# CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

"""Class to encapsulate the .cvsignore logic.

"""

__rcs_info__ = {
    #
    #  Creation Information
    #
    'module_name'  : '$RCSfile: cvsignore.py,v $',
    'rcs_id'       : '$Id: cvsignore.py,v 1.1 2003/01/18 14:15:29 doughellmann Exp $',
    'creator'      : 'Doug Hellmann <doug@hellfly.net>',
    'project'      : 'HappyDoc',
    'created'      : 'Sat, 18-Jan-2003 08:29:30 EST',

    #
    #  Current Information
    #
    'author'       : '$Author: doughellmann $',
    'version'      : '$Revision: 1.1 $',
    'date'         : '$Date: 2003/01/18 14:15:29 $',
}
try:
    __version__ = __rcs_info__['version'].split(' ')[1]
except:
    __version__ = '0.0'

#
# Import system modules
#
from fnmatch import fnmatch
import os

#
# Import Local modules
#


#
# Module
#

class CVSIgnoreList:

    def __init__(self, ignoreFileNames):
        self.patterns = []
        
        self.ignore_filenames = ignoreFileNames

        for filename in self.ignore_filenames:
            self.readIgnoreFile(filename)
        return

    def readIgnoreFile(self, filename):
        """Reads a .cvsignore file to include its patterns with the patterns
        read from other files.
        """
        try:
            file = open(filename, 'rt')
        except IOError:
            return
        else:
            lines = file.readlines()
            file.close()
            patterns = [ s.strip() for s in lines ]
            
            for pattern in patterns:
                if pattern not in self.patterns:
                    self.patterns.append(pattern)
        return

    def shouldIgnoreFile(self, filename):
        """Returns true if the filename should be ignored.
        """
        basename = os.path.basename(filename)
        for pattern in self.patterns:
            if basename == pattern:
                return 1
            elif fnmatch(basename, pattern):
                return 1
        return 0

    
