#!/usr/bin/env python
#
# $Id: templatefile.py,v 1.2 2003/03/16 16:28:12 doughellmann Exp $
#
# Copyright 2003 Doug Hellmann.
#
#
#                         All Rights Reserved
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby
# granted, provided that the above copyright notice appear in all
# copies and that both that copyright notice and this permission
# notice appear in supporting documentation, and that the name of Doug
# Hellmann not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission.
#
# DOUG HELLMANN DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
# NO EVENT SHALL DOUG HELLMANN BE LIABLE FOR ANY SPECIAL, INDIRECT OR
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
# OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
# NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
# CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

"""Class to manage a single template file.

"""

__rcs_info__ = {
    #
    #  Creation Information
    #
    'module_name'  : '$RCSfile: templatefile.py,v $',
    'rcs_id'       : '$Id: templatefile.py,v 1.2 2003/03/16 16:28:12 doughellmann Exp $',
    'creator'      : 'Doug Hellmann <doug@hellfly.net>',
    'project'      : 'HappyDoc',
    'created'      : 'Sun, 26-Jan-2003 13:20:27 EST',

    #
    #  Current Information
    #
    'author'       : '$Author: doughellmann $',
    'version'      : '$Revision: 1.2 $',
    'date'         : '$Date: 2003/03/16 16:28:12 $',
}
try:
    __version__ = __rcs_info__['version'].split(' ')[1]
except:
    __version__ = '0.0'

#
# Import system modules
#
from cStringIO import StringIO
from pprint import pprint


#
# Import Local modules
#
from happydoclib.docset.docset_TAL import hdExpressions
from happydoclib.docset.docset_TAL.TAL.HTMLTALParser import HTMLTALParser
from happydoclib.docset.docset_TAL.TAL.TALGenerator import TALGenerator
from happydoclib.docset.docset_TAL.TAL.TALInterpreter import TALInterpreter

from happydoclib.trace import trace

#
# Module
#
TRACE_LEVEL = 2

class TemplateFile:
    """Class to manage a single TAL template file.
    """

    def __init__(self, fileName, templateSet=None):
        """Constructor.
        """
        trace.into('TemplateFile', '__init__',
                   fileName=fileName,
                   outputLevel=TRACE_LEVEL,
                   )
        self.filename = fileName
        self.template_set = templateSet
        
        self.engine = hdExpressions.getEngine()
        self.generator = TALGenerator(self.engine, xml=0)
        self.parser = HTMLTALParser(self.generator)
        self.parser.parseFile(self.filename)
        self.program, self.macros = self.parser.getCode()
        #print 'PROGRAM:', self.program
        #print 'MACROS:'
        #pprint(self.macros)
        trace.outof(outputLevel=TRACE_LEVEL)
        return

    def getContext(self):
        c = {'template': self,
             'options': {},
             'nothing': None,
             'modules': hdExpressions.DocsetModuleImporter,
             }
        if self.template_set is not None:
            c['templates'] = self.template_set
        return c

    def render(self, extraContext={}):
        c = self.getContext()
        c.update(extraContext)
        context = self.engine.getContext(c)

        output = StringIO()
        
        interpreter = TALInterpreter(self.program,
                                     self.macros,
                                     context,
                                     output,
                                     wrap=0,
                                     )
        interpreter()
    
        return output.getvalue()

