#!/usr/bin/env python
#
# $Id: parsers_directory.py,v 1.1 2003/01/18 20:01:52 doughellmann Exp $
#
# Copyright 2003 Doug Hellmann.
#
#
#                         All Rights Reserved
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby
# granted, provided that the above copyright notice appear in all
# copies and that both that copyright notice and this permission
# notice appear in supporting documentation, and that the name of Doug
# Hellmann not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission.
#
# DOUG HELLMANN DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
# NO EVENT SHALL DOUG HELLMANN BE LIABLE FOR ANY SPECIAL, INDIRECT OR
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
# OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
# NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
# CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

"""Directory "parser".

"""

__rcs_info__ = {
    #
    #  Creation Information
    #
    'module_name'  : '$RCSfile: parsers_directory.py,v $',
    'rcs_id'       : '$Id: parsers_directory.py,v 1.1 2003/01/18 20:01:52 doughellmann Exp $',
    'creator'      : 'Doug Hellmann <doug@hellfly.net>',
    'project'      : 'HappyDoc',
    'created'      : 'Sat, 18-Jan-2003 12:03:15 EST',

    #
    #  Current Information
    #
    'author'       : '$Author: doughellmann $',
    'version'      : '$Revision: 1.1 $',
    'date'         : '$Date: 2003/01/18 20:01:52 $',
}
try:
    __version__ = __rcs_info__['version'].split(' ')[1]
except:
    __version__ = '0.0'

#
# Import system modules
#


#
# Import Local modules
#
from base import ParserBase

from happydoclib.packagetree import PackageTree
from happydoclib.trace import trace
from happydoclib.utils import extractSummary

#
# Module
#

TRACE_LEVEL=3

class Directory(PackageTree):
    """Directory node for directories and folders.
    """

    DOCSTRING_SOURCES = ('__init__.py', 'README.txt', 'README.stx')

    def getDocStringAndFormat(self):
        """Returns a tuple containing the actual documentation string
        and the format of that docstring as understood by the
        docstring converters plugins.
        """
        docstring = ''
        format = 'StructuredText'
        
        if hasattr(self, '_docstring'):
            docstring, format = self._docstring
            return (docstring, format)
        
        for name in self.DOCSTRING_SOURCES:
            trace.write('looking for %s' % name, outputLevel=TRACE_LEVEL)
            node = self.get(name)
            if node is not None:
                docstring, format = node.getDocStringAndFormat()
                self._docstring_source = node
                break

        self._docstring = (docstring, format)
        return (docstring, format)

    def getSummaryAndFormat(self):
        """Returns a tuple containing a one line summary of the
        documentation for the node and the format of that
        string as understood by the docstring converter plugins.
        """
        if hasattr(self, '_docstring_source'):
            return self._docstring_source.getSummaryAndFormat()
        
        elif hasattr(self, '_summary'):
            return (self._summary, self._docstring[1])
        
        docstring, format = self.getDocStringAndFormat()
        summary = extractSummary(docstring)
        self._summary = summary
        return (summary, format)
    
class DirectoryParser(ParserBase):
    """Parser for directories.
    """

    def __call__(self, parent, name):
        return Directory(parent, name)

def entryPoint():
    return { 'mimetype':'application/x-directory',
             'factory':DirectoryParser,
             }
