#!/usr/bin/env python
#
# $Id: test_cvsignore.py,v 1.1 2003/01/18 14:15:29 doughellmann Exp $
#
# Copyright 2003 Doug Hellmann.
#
#
#                         All Rights Reserved
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby
# granted, provided that the above copyright notice appear in all
# copies and that both that copyright notice and this permission
# notice appear in supporting documentation, and that the name of Doug
# Hellmann not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission.
#
# DOUG HELLMANN DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
# NO EVENT SHALL DOUG HELLMANN BE LIABLE FOR ANY SPECIAL, INDIRECT OR
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
# OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
# NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
# CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

"""Test the cvsignore module.

"""

__rcs_info__ = {
    #
    #  Creation Information
    #
    'module_name'  : '$RCSfile: test_cvsignore.py,v $',
    'rcs_id'       : '$Id: test_cvsignore.py,v 1.1 2003/01/18 14:15:29 doughellmann Exp $',
    'creator'      : 'Doug Hellmann <doug@hellfly.net>',
    'project'      : 'HappyDoc',
    'created'      : 'Sat, 18-Jan-2003 08:42:33 EST',

    #
    #  Current Information
    #
    'author'       : '$Author: doughellmann $',
    'version'      : '$Revision: 1.1 $',
    'date'         : '$Date: 2003/01/18 14:15:29 $',
}
try:
    __version__ = __rcs_info__['version'].split(' ')[1]
except:
    __version__ = '0.0'

#
# Import system modules
#
import unittest

#
# Import Local modules
#
from happydoclib.cvsignore import CVSIgnoreList

#
# Module
#

class CVSIgnoreTest(unittest.TestCase):

    def testIgnoreExplicitName(self):
        ignore_list = CVSIgnoreList([])
        ignore_list.patterns.append('foo.txt')
        if not ignore_list.shouldIgnoreFile('/tmp/foo.txt'):
            self.fail('Should have ignored foo.txt.')
        return
    
    def testIgnorePatternWithExt(self):
        ignore_list = CVSIgnoreList([])
        ignore_list.patterns.append('*.pyc')
        if not ignore_list.shouldIgnoreFile('/tmp/foo.pyc'):
            self.fail('Should have ignored foo.pyc.')
        return
    
    def testNotIgnoreExplicitName(self):
        ignore_list = CVSIgnoreList([])
        if ignore_list.shouldIgnoreFile('/tmp/foo.txt'):
            self.fail('Should not have ignored foo.txt.')
        return
    
    def testNotIgnorePatternWithExt(self):
        ignore_list = CVSIgnoreList([])
        if ignore_list.shouldIgnoreFile('/tmp/foo.pyc'):
            self.fail('Should not have ignored foo.pyc.')
        return


