#!/usr/bin/env python
#
# $Id: test_scanner.py,v 1.11 2003/01/01 13:49:11 doughellmann Exp $
#
# Copyright 2002 Doug Hellmann.
#
#
#                         All Rights Reserved
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby
# granted, provided that the above copyright notice appear in all
# copies and that both that copyright notice and this permission
# notice appear in supporting documentation, and that the name of Doug
# Hellmann not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission.
#
# DOUG HELLMANN DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
# NO EVENT SHALL DOUG HELLMANN BE LIABLE FOR ANY SPECIAL, INDIRECT OR
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
# OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
# NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
# CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

"""Tests for the scanner module.

"""

__rcs_info__ = {
    #
    #  Creation Information
    #
    'module_name'  : '$RCSfile: test_scanner.py,v $',
    'rcs_id'       : '$Id: test_scanner.py,v 1.11 2003/01/01 13:49:11 doughellmann Exp $',
    'creator'      : 'Doug Hellmann <doug@hellfly.net>',
    'project'      : 'HappyDoc',
    'created'      : 'Sat, 16-Nov-2002 17:45:13 EST',

    #
    #  Current Information
    #
    'author'       : '$Author: doughellmann $',
    'version'      : '$Revision: 1.11 $',
    'date'         : '$Date: 2003/01/01 13:49:11 $',
}
try:
    __version__ = __rcs_info__['version'].split(' ')[1]
except:
    __version__ = '0.0'

#
# Import system modules
#
import os
import unittest

#
# Import Local modules
#
from happydoclib.scanner import Scanner


#
# Module
#

class ScannerTestCase(unittest.TestCase):

    def testCreateScannerWithoutInputDirectory(self):
        try:
            scanner = Scanner()
        except TypeError:
            pass
        else:
            self.fail('Did not raise TypeError')
        return

    def testScannerNoSuchDir(self):
        try:
            scanner = Scanner(['TestCases/testScannerNotThere'])
        except ValueError:
            pass
        else:
            self.fail('Did not raise value error for missing directory.')
        return        

    def testScannerOnTestCase(self):
        scanner = Scanner(['TestCases/testScanner'])

        trees = scanner.getPackageTrees()

        self.failUnless(trees, 'No package trees.')

        self.failUnless(len(trees) == 1,
                        'Not the right number of trees (%s)' % len(trees))

        expected_tree = trees[0]
        self.failUnlessEqual(
            expected_tree.getName(),
            'testScanner',
            'First level tree got %s instead of testScanner' % expected_tree.getName(),
            )

        level_one = expected_tree['levelOne']
        self.failUnlessEqual(
            level_one.getName(),
            'levelOne',
            'First sub-level tree got %s instead of levelOne' % level_one.getName(),
            )

        #
        # Try for one.py
        #
        try:
            module_one = level_one['one.py']
        except KeyError:
            self.fail('Could not get one.py')
        else:
            #
            # Make sure of the mimetype
            #
            mimetype = module_one.getMimeType()
            self.failUnlessEqual(mimetype, ('text/x-python', None))

        #
        # Try for class in one.py
        #
        try:
            class_one = module_one['One']
        except KeyError:
            self.fail('Could not get One')
        else:
            #
            # Make sure of the mimetype
            #
            mimetype = class_one.getMimeType()
            self.failUnlessEqual(mimetype, ('application/x-class', None))

        #
        # Try for README.txt
        #
        try:
            readme = level_one['README.txt']
        except KeyError:
            self.fail('Could not get README.txt')
        else:
            #
            # Make sure of the mimetype
            #
            mimetype = readme.getMimeType()
            self.failUnlessEqual(mimetype, ('text/plain', None))

        #
        # Try for Existing.html
        #
        try:
            readme = level_one['Existing.html']
        except KeyError:
            self.fail('Could not get Existing.html')
        else:
            #
            # Make sure of the mimetype
            #
            mimetype = readme.getMimeType()
            self.failUnlessEqual(mimetype, ('text/html', None))

        #
        # Make sure we are *not* ignoring
        #
        try:
            module_one = level_one['ignoreme.py']
        except KeyError:
            self.fail('Could not get ignoreme.py')

        #
        # Look for levelTwo
        #
        level_two = level_one['levelTwo']
        self.failUnlessEqual(
            level_two.getName(),
            'levelTwo',
            'Second sub-level tree got %s instead of levelTwo' % level_two.getName(),
            )

        #
        # Try for two.py
        #
        try:
            module_two = level_two['two.py']
        except KeyError:
            self.fail('Could not get two.py')

            
        #
        # Try for README.txt
        #
        try:
            readme = level_two['README.stx']
        except KeyError:
            self.fail('Could not get README.stx')
        else:
            #
            # Make sure of the mimetype
            #
            mimetype = readme.getMimeType()
            self.failUnlessEqual(mimetype, ('text/x-structured', None))
            
        return

if __name__ == '__main__':
    unittest.main()
    
